package com.glxp.udidl.admin.controller.udchs;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.glxp.udidl.admin.dto.sys.CodeRelRequest;
import com.glxp.udidl.admin.dto.udchs.BaseParam;
import com.glxp.udidl.admin.entity.chs.CodeRel;
import com.glxp.udidl.admin.entity.chs.YbDrug;
import com.glxp.udidl.admin.entity.collect.ThirdAliDrug;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.chs.CodeRelVo;
import com.glxp.udidl.admin.service.collect.ThirdAliDrugService;
import com.glxp.udidl.admin.service.udchs.impl.CodeRelServiceImpl;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;


/**
 * 多码融合库
 */
@Api(tags = "多码融合库信息接口")
@RestController
@RequestMapping("/udchs/codeRel")
public class CodeRelController {

    @Autowired
    private CodeRelServiceImpl codeRelService;
    @Resource
    private ThirdAliDrugService thirdAliDrugService;

    @ApiOperation(value = "获取指定信息", response = CodeRel.class)
    @PostMapping("/list")
    public BaseResponse getList(@RequestBody CodeRel param) {
        if ( param == null ||
                (StrUtil.isBlank(param.getDiNameCode()) && StrUtil.isBlank(param.getDrugCode()))
        ){
            return ResultVOUtils.error(ResultEnum.DATA_ERROR,"参数错误");
        }
        List<CodeRel> list = codeRelService.list(new LambdaQueryWrapper<CodeRel>()
                .eq(CodeRel::getDiNameCode,param.getDiNameCode()).or().eq(CodeRel::getDrugCode,param.getDrugCode()));
        return ResultVOUtils.success(list);
    }


    @ApiOperation(value = "更新指定信息", response = CodeRel.class)
    @PostMapping("/update")
    public BaseResponse updateByCodeRel(@RequestBody CodeRel param) {
        if ( param == null ||
                (StrUtil.isBlank(param.getDiNameCode()) && StrUtil.isBlank(param.getDrugCode()))
        ){
            return ResultVOUtils.error(ResultEnum.DATA_ERROR,"参数错误");
        }
        boolean b = codeRelService.updateByCodeRel(param);
        if (!b){
            return ResultVOUtils.error(500,"更新失败");
        }
        return ResultVOUtils.success("更新成功");
    }


//    @ApiOperation(value = "更新指定信息", response = List.class)
    @PostMapping("/updateBatch")
    public BaseResponse updateBatchByCodeRel(@RequestBody List<CodeRel> paramList) {
        if (CollUtil.isEmpty(paramList)){
            return ResultVOUtils.error(500,"更新失败,数据为空");
        }
        codeRelService.updateBatchByCodeRel(paramList);
        return ResultVOUtils.success("更新成功");
    }


    @ApiOperation(value = "获取指定信息", response = CodeRel.class)
    @PostMapping("/listAndAliDrug")
    public BaseResponse getListAndAliDrug(@RequestBody CodeRel param) {
        if ( param == null ||
                (StrUtil.isBlank(param.getYbbm()) && StrUtil.isBlank(param.getYbbm()))
        ){
            return ResultVOUtils.error(ResultEnum.DATA_ERROR,"参数错误");
        }

//        List<CodeRelVo> respList = new ArrayList<>();
        List<ThirdAliDrug> thirdAliDrugList = new ArrayList<>();

        List<CodeRel> list = codeRelService.list(new LambdaQueryWrapper<CodeRel>()
                .eq(CodeRel::getYbbm,param.getYbbm()));

        if (CollUtil.isNotEmpty(list)){
            for (int i = 0; i < list.size(); i++) {
                CodeRel codeRel = list.get(i);
                CodeRelVo codeRelVo = new CodeRelVo();
                BeanUtils.copyProperties(codeRel,codeRelVo);

                String drugCode = codeRelVo.getDrugCode();
                if (StrUtil.isNotEmpty(drugCode)){
                    //本地查找 未找到再到阿里查找
                    ThirdAliDrug thirdAliDrug = thirdAliDrugService.getThirdAliDrug(drugCode);
                    thirdAliDrug.setPackUnit(codeRel.getPackUnit());
                    thirdAliDrug.setBhxjsl(codeRel.getBhxjsl());
                    thirdAliDrug.setXjdw(codeRel.getXjdw());
                    thirdAliDrug.setYbBzgg(codeRel.getYbBzgg());
                    thirdAliDrug.setDmcx(true);
//                    codeRelVo.setThirdAliDrug(thirdAliDrug);
                    thirdAliDrugList.add(thirdAliDrug);
                }
//                respList.add(codeRelVo);
            }
        }

        return ResultVOUtils.success(thirdAliDrugList);
    }
    @ApiOperation(value = "更新指定信息", response = CodeRel.class)
    @PostMapping("/delete")
    public BaseResponse delete(@RequestBody CodeRel param) {
        if ( param == null ||
                (StrUtil.isBlank(param.getYbbm()) && StrUtil.isBlank(param.getYbbm()))
        ){
            return ResultVOUtils.error(ResultEnum.DATA_ERROR,"参数错误");
        }
        boolean b = codeRelService.remove(
                new QueryWrapper<CodeRel>().eq("drugCode",param.getDrugCode()).eq("ybbm",param.getYbbm())
        );
        if (!b){
            return ResultVOUtils.error(500,"删除失败");
        }
        return ResultVOUtils.success("删除失败");
    }
}
