package com.glxp.udidl.admin.service.chsapi;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.udidl.admin.res.chs.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.*;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

@Slf4j
@Service
public class ChsYbChsHttpClient {

    /**
     * 获取耗材分类目录
     */
    String getHcflUrl = "https://code.nhsa.gov.cn/hc/stdSpecification/getStdSpecificationListDataCompanyReport.html";
    /**
     * 体外诊断试剂信息
     */
    String getTwzdUrl = "https://code.nhsa.gov.cn/sj/publish/cms_sj/product.html";
    /**
     * 获取医用耗材信息
     */
    String getHcxxUrl = "https://code.nhsa.gov.cn/hc/stdPublishData/getNewPublishRelationDataList.html";
    /**
     * 获取医用耗材规格信息
     */
    String getHcxxGgUrl = "https://code.nhsa.gov.cn/hc/stdYgbData/getPublicHcDataList.html";
    /**
     * 获取医用耗材规格信息
     */
    String getDrugUrl = "https://code.nhsa.gov.cn/yp/stdGoodsPublic/getStdGoodsPublicData.html";


    //获取HCF列表
    public YbHcflResponse getYbHcflList(Integer pageNum, Integer pageSize) {
        Map<String, Object> params = new HashMap<>();
        params.put("_search",false);
        params.put("nd",System.currentTimeMillis());
        params.put("rows", pageSize);
        params.put("page", pageNum);
        params.put("sord", "asc");

        String response = sendPost(getHcflUrl+getParams(params), "",Charset.forName("utf-8"));
        YbHcflResponse baseResponse = JSONObject.parseObject(response, new TypeReference<YbHcflResponse>() {
        });
        return baseResponse;
    }

    //获取HCF列表
    public YbTwzdResponse getYbTwzdList(Integer pageNum, Integer pageSize) {
        Map<String, Object> params = new HashMap<>();
        params.put("_search",false);
        params.put("nd",System.currentTimeMillis());
        params.put("rows", pageSize);
        params.put("page", pageNum);
        params.put("sord", "asc");
        params.put("code", "C");

        try {
            Thread.sleep(15000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        String response = sendPost(getTwzdUrl+getParams(params), "",Charset.forName("utf-8"));
        YbTwzdResponse baseResponse = JSONObject.parseObject(response, new TypeReference<YbTwzdResponse>() {
        });
        return baseResponse;
    }
    //获取HCF列表
    public YbDrugResponse getYbDrugList(Integer pageNum, Integer pageSize,String goodsCode) {
        Map<String, Object> params = new HashMap<>();
        params.put("_search",false);
        params.put("nd",System.currentTimeMillis());
        params.put("rows", pageSize);
        params.put("page", pageNum);
        if (StrUtil.isNotBlank(goodsCode)) params.put("goodsCode", goodsCode);


        String response = sendPost(getDrugUrl+getParams(params), "",Charset.forName("utf-8"));
        response = response.replace("”", "").replace("“", "").replace("\\\"" , "").replace("\\\"" , "").replace("\\", "");
        System.out.println(response);
        YbDrugResponse baseResponse = JSONObject.parseObject(response, new TypeReference<YbDrugResponse>() {
        });
        return baseResponse;
    }

    //获取Hcxx列表
    public YbHcxxResponse getYbHcxxList(Integer pageNum, Integer pageSize) {
        Map<String, Object> params = new HashMap<>();
        params.put("_search",false);
        params.put("nd",System.currentTimeMillis());
        params.put("rows", pageSize);
        params.put("page", pageNum);
        params.put("sord", "asc");
        params.put("specificationCode", "C");
        try {
            Thread.sleep(15000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        String response = sendPost(getHcxxUrl+getParams(params), "",Charset.forName("utf-8"));
        YbHcxxResponse baseResponse = JSONObject.parseObject(response, new TypeReference<YbHcxxResponse>() {
        });
        return baseResponse;
    }

    //获取HcxxGg列表
    public YbHcxxGgResponse getYbHcxxGgList(String specificationCode,Integer pageNum, Integer pageSize) {
        Map<String, Object> params = new HashMap<>();
        params.put("_search",false);
        params.put("nd",System.currentTimeMillis());
        params.put("rows", pageSize);
        params.put("page", pageNum);
        params.put("sord", "asc");
        params.put("specificationCode", specificationCode);
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        String response = sendPost(getHcxxGgUrl+getParams(params), "",Charset.forName("utf-8"));
        YbHcxxGgResponse baseResponse = JSONObject.parseObject(response, new TypeReference<YbHcxxGgResponse>() {
        });
        return baseResponse;
    }

//    public static void main(String[] args) {
//        YbChsHttpClient ybChsHttpClient = new YbChsHttpClient();
//        YbHcxxGgResponse ybHcflList = ybChsHttpClient.getYbHcxxGgList("C0101010010100103941",1, 200);
//        System.out.println(ybHcflList);
//    }


    /**
     * @param paramsMap
     * @return
     */
    private static String getParams(Map<String, Object> paramsMap) {
        String params = "?";
        Iterator<String> iterator = paramsMap.keySet().iterator();
        while(iterator.hasNext()) {
            String key = iterator.next();
            params += key + "=" + paramsMap.get(key) + "&";
        }
        params = params.substring(0, params.length() - 1);
        return params;
    }


    public static String sendPost(String url, String param, Charset charset) {
        try {
            URL httpurl = new URL(url);
            //打开和url的连接
            HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
            //设置通用的请求头属性
            httpConn.setRequestMethod("POST");
            httpConn.setRequestProperty("accept", "*/*");
            httpConn.setRequestProperty("connection", "Keep-Alive");
            httpConn.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            httpConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=" + charset.name());
            /*httpConn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; "
                    + "Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)");*/

            // 发送POST请求必须设置如下两行   否则会抛异常（java.net.ProtocolException: cannot write to a URLConnection if doOutput=false - call setDoOutput(true)）
            httpConn.setDoOutput(true);
            httpConn.setDoInput(true);

            httpConn.setUseCaches(false);
            /*httpConn.setRequestMethod("POST");
            httpConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=" + charset.name());*/
            httpConn.connect();
            OutputStream outputStream = httpConn.getOutputStream();
            PrintWriter out = new PrintWriter(outputStream);
            //print在方法内也是调用的write方法，多了null的判断逻辑
            out.print(param);
            //out.write(param);
            out.flush();
            out.close();
            InputStream inputStream = httpConn.getInputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset.name()));
            StringBuffer stringBuffer = new StringBuffer();
            String line = null;
            while ((line = bufferedReader.readLine()) != null) {
                stringBuffer.append(line);
            }
            String resp = stringBuffer.toString();
            // System.out.println(resp);
            if (stringBuffer != null) {
                try {
                    bufferedReader.close();
                } catch (IOException var18) {
                    var18.printStackTrace();
                }
            }
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException var17) {
                    var17.printStackTrace();
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException var17) {
                    var17.printStackTrace();
                }
            }
            return resp;
        } catch (IOException e) {
            throw new RuntimeException(String.format("url:%s,param:%s,message:%s", url, param, e.getMessage()), e);
        }
    }
}
