package com.glxp.udidl.admin.service.collect;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.glxp.udidl.admin.dao.collect.ThirdAliDrugMapper;
import com.glxp.udidl.admin.entity.auth.AuthAliUser;
import com.glxp.udidl.admin.entity.collect.ThirdAliDrug;
import com.glxp.udidl.admin.exception.JsonException;
import com.glxp.udidl.admin.req.alihealth.AlihealthKytDrugrescodeReqeust;
import com.glxp.udidl.admin.req.alihealth.local.AlihealthThirdAliDrugInsertReqeust;
import com.glxp.udidl.admin.req.collect.ThirdAliDrugAddRequest;
import com.glxp.udidl.admin.req.collect.ThirdAliDrugRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.auth.AuthAliUserService;
import com.glxp.udidl.admin.util.alihealth.AlihealthUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class ThirdAliDrugService extends ServiceImpl<ThirdAliDrugMapper, ThirdAliDrug> {


    public List<ThirdAliDrug> filterList(ThirdAliDrugRequest thirdAliDrug) {
        if (thirdAliDrug == null) {
            return Collections.emptyList();
        }
        if (thirdAliDrug.getPage() != null) {
            int offset = (thirdAliDrug.getPage() - 1) * thirdAliDrug.getLimit();
            PageHelper.offsetPage(offset, thirdAliDrug.getLimit());
        }
        return this.baseMapper.filterList(thirdAliDrug);
    }


    public List<ThirdAliDrug> findByByApproved(String approvalNum) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<ThirdAliDrug>().eq(ThirdAliDrug::getApprovalNum, approvalNum));
    }

    public void addThirdAliDrug(ThirdAliDrugAddRequest relCodeBatchRequest) {
        List<ThirdAliDrug> list = relCodeBatchRequest.getList();
        if (CollUtil.isNotEmpty(list)){
            List<String> nameCodes = list.stream().map(ThirdAliDrug::getNameCode).collect(Collectors.toList());
            List<String> updateNameCodes = new ArrayList<>();
            List<ThirdAliDrug> drugs = this.list(new LambdaQueryWrapper<ThirdAliDrug>().in(ThirdAliDrug::getNameCode, nameCodes));
            if (CollUtil.isNotEmpty(drugs)){
                updateNameCodes = drugs.stream().map(ThirdAliDrug::getNameCode).collect(Collectors.toList());
            }
            List<ThirdAliDrug> addList = new ArrayList<>();
            List<String> finalUpdateNameCodes = updateNameCodes;
            list.forEach(item -> {
                if (finalUpdateNameCodes.size() > 0 ){
                    if (!finalUpdateNameCodes.contains(item.getNameCode())){
                        item.setId(null);
                        addList.add(item);
                    }
                }else {
                    item.setId(null);
                    addList.add(item);
                }
            });

            if (CollUtil.isNotEmpty(addList)){
                this.baseMapper.saveOrUpdateBatch(addList);
            }

        }
    }
    @Resource
    private AlihealthUtils alihealthUtils;
    @Resource
    private AuthAliUserService authAliUserService;

    public void updateThirdAliDrugTask() {
        //1、读取 任务待定表 如果有数据 继续执行 如果没有 读取数据 进行任务的插入
        List<ThirdAliDrug> groupedByCpmctymcTask = this.baseMapper.groupByCpmctymcTask();
        
        if (CollUtil.isEmpty(groupedByCpmctymcTask)){
            List<ThirdAliDrug> groupedByCpmctymc = this.baseMapper.groupByCpmctymc();
            if (CollUtil.isEmpty(groupedByCpmctymc))return;
            this.insertBatchInChunks(groupedByCpmctymc);
            groupedByCpmctymcTask = groupedByCpmctymc;
        }
        

        if (CollUtil.isNotEmpty(groupedByCpmctymcTask)){
            for (int i = 0; i < groupedByCpmctymcTask.size(); i++) {
                ThirdAliDrug thirdAliDrug = groupedByCpmctymcTask.get(i);
                String cpmctymc = thirdAliDrug.getCpmctymc();

                ThirdAliDrugAddRequest thirdAliDrugAddRequest = new ThirdAliDrugAddRequest();
                // TODO: 2025/1/4 这里用调用的 入参 cpmctymc
                List<ThirdAliDrug> add  = null;
                AlihealthThirdAliDrugInsertReqeust request  = new AlihealthThirdAliDrugInsertReqeust();
                AlihealthKytDrugrescodeReqeust alihealthKytDrugrescodeReqeust = new AlihealthKytDrugrescodeReqeust();
                alihealthKytDrugrescodeReqeust.setPhysic_name(cpmctymc);
                request.setAlihealthKytDrugrescodeReqeust(alihealthKytDrugrescodeReqeust);
                request.setIsAutoUpdate(true);

//                // key组装
//                AuthAliUser authAliUser = authAliUserService.getOne();
//
//                //增加次数限制
//                Integer drugResCodeCount = authAliUser.getDrugResCodeCount();
//                Integer drugResCodeCountMax = authAliUser.getDrugResCodeCountMax();
//                if (drugResCodeCount >= drugResCodeCountMax){
//                    throw new JsonException("当前系统调用阿里药品数据查询接口最大次数已达到，禁止访问，如需访问请联系管理员增加访问次数！");
//                }
//                drugResCodeCount = drugResCodeCount + 1;
//                authAliUser.setDrugResCodeCount(drugResCodeCount);
//                authAliUserService.updateById(authAliUser);

                try {
                    BaseResponse<List<ThirdAliDrug>> baseResponse = alihealthUtils.thirdAliDrugInsertAll(request);
                    if(baseResponse.getCode() == 20000){
                        add = baseResponse.getData();
                        //删除 已完成的任务
                        this.baseMapper.deleteCpmctymcTask(cpmctymc);

                    }else {
                        log.error("阿里访问失败==="+baseResponse.getMessage());
                    }
                }catch (Exception e){
                    log.error("阿里访问失败==="+e.getMessage());
                }

                if (CollUtil.isNotEmpty(add)){
                    thirdAliDrugAddRequest.setList(add);
                    this.addThirdAliDrug(thirdAliDrugAddRequest);
                }

                
                try {
                    Thread.sleep(15000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    public ThirdAliDrug getThirdAliDrug(String drugCode) {
        ThirdAliDrugRequest thirdAliDrug = new ThirdAliDrugRequest();
        thirdAliDrug.setNameCode(drugCode);
        List<ThirdAliDrug> thirdAliDrugs = this.baseMapper.filterList(thirdAliDrug);
        if (CollUtil.isNotEmpty(thirdAliDrugs)){
            return thirdAliDrugs.get(0);
        }else {
            // TODO: 2025/1/6 前往阿里查找 未找到的话 直接返回  new ThirdAliDrug()

        }
        return new ThirdAliDrug();
    }



    // 方法用于分批插入数据
    public void insertBatchInChunks(List<ThirdAliDrug> groupedByCpmctymc) {
        int batchSize = 200; // 每批插入200条记录
        int totalSize = groupedByCpmctymc.size();

        // 计算需要多少个批次
        int totalBatches = (totalSize + batchSize - 1) / batchSize;

        for (int i = 0; i < totalBatches; i++) {
            int fromIndex = i * batchSize;
            int toIndex = Math.min(fromIndex + batchSize, totalSize);

            List<ThirdAliDrug> batchList = groupedByCpmctymc.subList(fromIndex, toIndex);

            // 调用批量插入方法
            boolean result = this.baseMapper.insertBatchTask(batchList);
            // 这里可以添加逻辑来处理每个批次插入的结果，例如记录日志或处理错误
            if (!result) {
                // 处理插入失败的情况，例如抛出异常或记录错误日志
                System.err.println("Batch insertion failed for batch starting at index " + fromIndex);
                // 根据需要决定是否继续执行后续的批次插入
            }
        }
    }

    public List<ThirdAliDrug> updateByCpmctymc(String cpmctymc) {

        ThirdAliDrugAddRequest thirdAliDrugAddRequest = new ThirdAliDrugAddRequest();
        // TODO: 2025/1/4 这里用调用的 入参 cpmctymc
        List<ThirdAliDrug> add  = null;
        AlihealthThirdAliDrugInsertReqeust request  = new AlihealthThirdAliDrugInsertReqeust();
        AlihealthKytDrugrescodeReqeust alihealthKytDrugrescodeReqeust = new AlihealthKytDrugrescodeReqeust();
        alihealthKytDrugrescodeReqeust.setPhysic_name(cpmctymc);
        request.setAlihealthKytDrugrescodeReqeust(alihealthKytDrugrescodeReqeust);
        request.setIsAutoUpdate(true);

//        // key组装
//        AuthAliUser authAliUser = authAliUserService.getOne();
//
//        //增加次数限制
//        Integer drugResCodeCount = authAliUser.getDrugResCodeCount();
//        Integer drugResCodeCountMax = authAliUser.getDrugResCodeCountMax();
//        if (drugResCodeCount >= drugResCodeCountMax){
//            throw new JsonException("当前系统调用阿里药品数据查询接口最大次数已达到，禁止访问，如需访问请联系管理员增加访问次数！");
//        }
//        drugResCodeCount = drugResCodeCount + 1;
//        authAliUser.setDrugResCodeCount(drugResCodeCount);
//        authAliUserService.updateById(authAliUser);

        try {
            BaseResponse<List<ThirdAliDrug>> baseResponse = alihealthUtils.thirdAliDrugInsertAll(request);
            if(baseResponse.getCode() == 20000){
                add = baseResponse.getData();
                //删除 已完成的任务
                this.baseMapper.deleteCpmctymcTask(cpmctymc);
            }else {
                log.error("阿里访问失败==="+baseResponse.getMessage());
                throw new JsonException(baseResponse.getMessage());
            }
        }catch (Exception e){
            log.error("阿里访问失败==="+e.getMessage());
            throw new JsonException("阿里访问失败");
        }

        if (CollUtil.isNotEmpty(add)){
            thirdAliDrugAddRequest.setList(add);
            this.addThirdAliDrug(thirdAliDrugAddRequest);
        }
        return add;
    }
}
